<?php

namespace Drupal\site_settings_domain\Hook;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;

/**
 * Provides form alter hooks.
 */
final class FormAlter {

  use StringTranslationTrait;

  /**
   * Constructs the hook class.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly RouteMatchInterface $routeMatch,
  ) {}

  #[Hook('form_site_setting_entity_form_alter')]
  public function siteSettingsFormAlter(array &$form, FormStateInterface $form_state): void {
    $form_object = $form_state->getFormObject();
    /** @var \Drupal\site_settings\SiteSettingEntityInterface $site_setting_entity */
    $site_setting_entity = $form_object->getEntity();
    if ($site_setting_entity->isNew() || $this->routeMatch->getRouteName() !== 'entity.site_setting_entity.edit_form') {
      return;
    }

    $domain_context_parent = $site_setting_entity->get('domain_context_parent')->entity
      ?: $site_setting_entity;

    $storage = $this->entityTypeManager->getStorage('site_setting_entity');
    $linked_setting_entities = $storage->loadByProperties([
      'domain_context_parent' => $domain_context_parent->id(),
    ]);
    $linked_settings = [];
    foreach ($linked_setting_entities as $linked_setting_entity) {
      $domain_id = $linked_setting_entity->get('domain_context')->target_id;
      $linked_settings[$domain_id] = $linked_setting_entity;
    }

    /** @var \Drupal\domain\DomainInterface $current_domain */
    $current_domain = $site_setting_entity->get('domain_context')->entity;

    $form['existing_domains'] = [
      '#type' => 'container',
      '#weight' => -2,
      // Add parameter so content translation doesn't add "all languages" to label.
      '#multilingual' => TRUE,
    ];

    $form['new_domains'] = [
      '#type' => 'container',
      '#weight' => -1,
      // Add parameter so content translation doesn't add "all languages" to label.
      '#multilingual' => TRUE,
    ];

    if ($domain_context_parent->get('domain_context')->isEmpty()) {
      $url = $domain_context_parent->toUrl('edit-form');
      if ($url->access()) {
        $form['existing_domains']['_fallback'] = [
          '#type' => 'link',
          '#title' => $this->t('Edit fallback'),
          '#url' => $url,
          '#attributes' => [
            'class' => array_filter([
              'button',
              'button--small',
              'button--' . (empty($current_domain) ? 'primary' : 'secondary'),
            ]),
          ],
          '#weight' => -1,
        ];
      }
    }
    else {
      $url = Url::fromRoute('site_settings_domain.clone_site_setting', [
        'site_setting_entity' => $domain_context_parent->id(),
      ]);
      if ($url->access()) {
        $form['new_domains']['_fallback'] = [
          '#type' => 'link',
          '#title' => $this->t('Add fallback'),
          '#url' => $url,
          '#attributes' => [
            'class' => array_filter([
              'button',
              'button--small',
              'button--secondary',
              'button--action',
            ]),
          ],
        ];
      }

      $url = $domain_context_parent->toUrl('edit-form');
      if ($url->access()) {
        $parent_domain = $domain_context_parent->get('domain_context')->entity;
        $form['existing_domains'][$parent_domain->id()] = [
          '#type' => 'link',
          '#title' => $this->t('@operation context for @domain', [
            '@operation' => 'Edit',
            '@domain' => $parent_domain->label(),
          ]),
          '#url' => $url,
          '#attributes' => [
            'class' => array_filter([
              'button',
              'button--small',
              'button--' . ($current_domain->id() === $parent_domain->id() ? 'primary' : 'secondary'),
            ]),
          ],
        ];
      }
    }

    $domain_storage = $this->entityTypeManager->getStorage('domain');
    foreach ($domain_storage->loadMultiple() as $domain_id => $domain) {
      if ($domain_id === $domain_context_parent->get('domain_context')->target_id) {
        continue;
      }

      $operation = isset($linked_settings[$domain_id]) ? 'edit' : 'add';
      $url = isset($linked_settings[$domain_id])
        ? $linked_settings[$domain_id]->toUrl('edit-form')
        : Url::fromRoute('site_settings_domain.clone_site_setting', [
            'site_setting_entity' => $domain_context_parent->id(),
            'domain' => $domain_id,
          ]);

      if (!$url->access()) {
        continue;
      }

      $form[$operation === 'add' ? 'new_domains' : 'existing_domains'][$domain_id] = [
        '#type' => 'link',
        '#title' => $this->t('@operation context for @domain', [
          '@operation' => ucfirst($operation),
          '@domain' => $domain->label(),
        ]),
        '#url' => $url,
        '#attributes' => [
          'class' => array_filter([
            'button',
            'button--small',
            'button--' . ($domain_id === $current_domain?->id() ? 'primary' : 'secondary'),
            $operation === 'add' ? 'button--action' : NULL,
          ]),
        ],
      ];
    }
  }

}
