<?php

namespace Drupal\site_settings_domain\Plugin\SiteSettingsLoader;

use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityTypeRepositoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\domain\DomainNegotiatorInterface;
use Drupal\site_settings\SiteSettingsLoaderBase;
use Drupal\site_settings\SiteSettingsLoaderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Load the site settings in a standard Drupal way with domain context.
 *
 * This loads the full site settings objects, respecting
 * the manage display settings. It is typically used
 * with twig filters for quick rendering in any template.
 *
 * @SiteSettingsLoader(
 *   id = "full_domain",
 *   label = @Translation("Full Site Settings Loader (domain context)"),
 *   autoload_by_default = FALSE
 * )
 */
class FullSiteSettingsLoader extends SiteSettingsLoaderBase implements SiteSettingsLoaderInterface {

  /**
   * Constructor.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param mixed $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityTypeRepositoryInterface $entityTypeRepository
   *   The entity type repository.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   * @param \Drupal\Core\Cache\CacheTagsInvalidatorInterface $cacheTagsInvalidator
   *   The cache tags invalidator.
   * @param \Drupal\domain\DomainNegotiatorInterface $domainNegotiator
   *   The domain negotiator.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityTypeRepositoryInterface $entityTypeRepository,
    protected LanguageManagerInterface $languageManager,
    protected CacheTagsInvalidatorInterface $cacheTagsInvalidator,
    protected DomainNegotiatorInterface $domainNegotiator,
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $entityTypeManager,
      $this->entityTypeRepository,
      $languageManager,
      $cacheTagsInvalidator,
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_type.repository'),
      $container->get('language_manager'),
      $container->get('cache_tags.invalidator'),
      $container->get('domain.negotiator'),
    );
  }

  /**
   * {@inheritDoc}
   */
  public function allowAutoload(): bool {
    return FALSE;
  }

  /**
   * Load site settings by group.
   *
   * @param string $group
   *   The name of the group.
   * @param string|null $langcode
   *   The language code.
   *
   * @return array
   *   All settings within the given group.
   */
  public function loadByGroup(string $group, ?string $langcode = NULL): array {
    $conditions = [
      [
        'field' => 'group',
        'value' => $group,
      ],
    ];

    return $this->loadByConditions($conditions, $langcode);
  }

  /**
   * Load site settings by group.
   *
   * @param string $type
   *   The name of the type.
   * @param string|null $langcode
   *   The language code.
   *
   * @return array
   *   All settings within the given group.
   */
  public function loadByType(string $type, ?string $langcode = NULL): array {
    $conditions = [
      [
        'field' => 'type',
        'value' => $type,
      ],
    ];

    return $this->loadByConditions($conditions, $langcode);
  }

  /**
   * Load site settings by group.
   *
   * @param bool $rebuild_cache
   *   Force rebuilding of the cache by setting to true.
   * @param string|null $langcode
   *   The language code.
   *
   * @return array
   *   All settings.
   */
  public function loadAll(bool $rebuild_cache = FALSE, ?string $langcode = NULL): array {
    return $this->loadByConditions([], $langcode);
  }

  /**
   * Load site settings by a given set of conditions.
   *
   * @param array $conditions
   *   The condition set.
   *
   * @return \Drupal\site_settings\SiteSettingEntityInterface[]
   *   All settings within the given group.
   */
  public function loadByConditions(array $conditions, ?string $langcode = NULL): array {
    // Get all site settings specific to this domain.
    $storage = $this->entityTypeManager->getStorage('site_setting_entity');
    $setting_entity_ids_query = $storage->getQuery()
      ->accessCheck(FALSE)
      ->condition('domain_context', $this->domainNegotiator->getActiveDomain()->id());

    if ($langcode) {
      $setting_entity_ids_query->condition('langcode', $langcode);
    }

    foreach ($conditions as $condition) {
      $setting_entity_ids_query->condition($condition['field'], $condition['value'], $condition['operator'] ?? '=');
    }

    $setting_entity_ids = $setting_entity_ids_query->execute();

    $setting_entities = $storage->loadMultiple($setting_entity_ids);
    $domain_context_parents = array_values(array_map(static function ($site_setting) {
      return $site_setting->get('domain_context_parent')->target_id ?: $site_setting->id();
    }, $setting_entities));

    // Retrieve non-specific site settings for settings where no specific one exists.
    $generic_setting_entity_ids_query = $storage->getQuery()
      ->accessCheck(FALSE)
      ->notExists('domain_context');

    if ($langcode) {
      $generic_setting_entity_ids_query->condition('langcode', $langcode);
    }

    foreach ($conditions as $condition) {
      $generic_setting_entity_ids_query->condition($condition['field'], $condition['value'], $condition['operator'] ?? '=');
    }

    if (!empty($domain_context_parents)) {
      $generic_setting_entity_ids_query
        ->condition('id', $domain_context_parents, 'NOT IN');
    }

    /** @var \Drupal\site_settings\SiteSettingEntityInterface[] $all_entities */
    $all_entities = $setting_entities + $storage->loadMultiple($generic_setting_entity_ids_query->execute());

    // Load language context if necessary.
    if ($langcode) {
      foreach ($all_entities as $id => $entity) {
        if ($entity->hasTranslation($langcode)) {
          $all_entities[$id] = $entity->getTranslation($langcode);
        }
      }
    }
    return $all_entities;
  }

  /**
   * Rebuild the cache.
   */
  public function rebuildCache($langcode): void {
    $this->clearCache();
    // Do nothing else, leave the rest to entity type manager.
  }

  /**
   * Clear the cache.
   */
  public function clearCache(): void {
    $this->cacheTagsInvalidator->invalidateTags([
      'site_setting_entity_list',
    ]);
  }

}
