<?php

namespace Drupal\site_settings_domain;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\domain_access\DomainAccessManagerInterface;
use Drupal\site_settings\SiteSettingEntityAccessControlHandler;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\VarDumper\VarDumper;

class SiteSettingEntityDomainAccessControlHandler extends SiteSettingEntityAccessControlHandler {

  /**
   * The domain access manager.
   *
   * @var \Drupal\domain_access\DomainAccessManagerInterface|null $domainAccessManager
   */
  protected ?DomainAccessManagerInterface $domainAccessManager = NULL;

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type): static {
    $instance = parent::createInstance($container, $entity_type);
    if ($container->get('module_handler')?->moduleExists('domain_access')) {
      $instance->domainAccessManager = $container->get('domain_access.manager');
    }
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account): AccessResultInterface {
    $access_result = parent::checkAccess($entity, $operation, $account);

    if ($operation === 'view') {
      return $access_result;
    }

    // Alter the delete flow.
    if ($operation === 'delete') {
      return $access_result->orIf(AccessResult::allowedIf($account->hasPermission('delete domain context specific site setting entities')));
    }

    // We don't have to do anything if domain_access isn't installed.
    if (empty($this->domainAccessManager)) {
      return $access_result;
    }

    // If user can publish to any domain, we can return early.
    if ($account->hasPermission('publish to any domain')) {
      return $access_result;
    }

    // If user did not pass the previous check, they shouldn't be allowed to edit the no-domain fallback.
    if (!$domain = $entity->get('domain_context')->entity) {
      return AccessResult::forbidden('You cannot publish to every domain.');
    }

    // Check if the editor has access to this domain.
    return $access_result->andIf(AccessResult::allowedIf($this->domainAccessManager->hasDomainPermissions(
      $account,
      $domain,
      ['publish to any assigned domain'],
    )));
  }

}
