<?php

namespace Drupal\site_settings_domain\Twig;

use Drupal\Component\Uuid\Uuid;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\TranslatableInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\domain\DomainNegotiatorInterface;
use Drupal\site_settings\Entity\SiteSettingEntity;
use Drupal\site_settings\SiteSettingEntityInterface;
use Drupal\site_settings\SiteSettingsLoaderPluginManager;
use Drupal\site_settings\Twig\TwigExtension as TwigExtensionBase;

/**
 * Twig extension to load site settings.
 *
 * This is primarily aimed at the full site settings loader,
 * but can be used by any loader.
 */
class TwigExtension extends TwigExtensionBase {

  /**
   * Constructs the Site Settings Twig Extension.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected SiteSettingsLoaderPluginManager $pluginManager,
    protected LanguageManagerInterface $languageManager,
    protected DomainNegotiatorInterface $domainNegotiator,
  ) {
    parent::__construct($entityTypeManager, $pluginManager, $this->languageManager);
  }

  /**
   * {@inheritDoc}
   */
  public function siteSetting(
    string $selector,
    string $view_mode = 'full',
    ?string $langcode = NULL,
    bool $check_access = TRUE,
  ): array {
    $storage = $this->entityTypeManager->getStorage('site_setting_entity');

    // Load by UUID, falling back to ID.
    if (Uuid::isValid($selector)) {
      $entities = $storage->loadByProperties(['uuid' => $selector]);
    }
    else {
      $entities = [$storage->load($selector)];
    }

    // Remove site setting entities that don't match the current domain context.
    $fallback_delta = NULL;
    $has_specific_context = FALSE;
    foreach ($entities as $delta => $entity) {
      if ($entity->get('domain_context')->isEmpty()) {
        $fallback_delta = $delta;
        continue;
      }

      $domain = $entity->get('domain_context')->entity;
      if ($domain->id() !== $this->domainNegotiator->getActiveDomain()->id()) {
        unset($entities[$delta]);
      }
      else {
        $has_specific_context = TRUE;
      }
    }

    // If a domain specific site setting is present, remove the fallback.
    if ($has_specific_context && $fallback_delta) {
      unset($entities[$fallback_delta]);
    }

    return $this->renderSiteSettings($entities, $view_mode, $langcode, $check_access, FALSE);
  }

  /**
   * {@inheritDoc}
   */
  public function siteSettingsByName(
    string $selector,
    string $view_mode = 'full',
    ?string $langcode = NULL,
    bool $check_access = TRUE,
  ): array {
    $storage = $this->entityTypeManager->getStorage('site_setting_entity');

    // Load by machine name.
    $site_settings = $storage->loadByProperties(['type' => $selector]);
    return $this->renderSiteSettings($site_settings, $view_mode, $langcode, $check_access);
  }

  /**
   * {@inheritDoc}
   */
  public function singleSiteSettingsByNameAndField(
    string $name,
    string $field_name,
    string $view_mode = 'full',
    ?string $langcode = NULL,
    bool $check_access = TRUE,
  ): array {
    $storage = $this->entityTypeManager->getStorage('site_setting_entity');

    // Load by machine name.
    $entities = $storage->loadByProperties(['type' => $name]);

    // Remove site setting entities that don't match the current domain context.
    $fallback_delta = NULL;
    $has_specific_context = FALSE;
    foreach ($entities as $delta => $entity) {
      if ($entity->get('domain_context')->isEmpty()) {
        $fallback_delta = $delta;
        continue;
      }

      $domain = $entity->get('domain_context')->entity;
      if ($domain->id() !== $this->domainNegotiator->getActiveDomain()->id()) {
        unset($entities[$delta]);
      }
      else {
        $has_specific_context = TRUE;
      }
    }

    // If a domain specific site setting is present, remove the fallback.
    if ($has_specific_context && $fallback_delta) {
      unset($entities[$fallback_delta]);
    }

    if (!empty($entities) && reset($entities) instanceof SiteSettingEntity) {

      // Render the entities in the specified view mode and
      // language code.
      /** @var \Drupal\site_settings\SiteSettingEntityInterface $site_setting */
      $site_setting = reset($entities);
      if ($site_setting->hasField($field_name)) {
        $langcode = $langcode ?? $this->languageManager->getCurrentLanguage()->getId();
        if ($site_setting->hasTranslation($langcode)) {
          $site_setting = $site_setting->getTranslation($langcode);
        }
        if ($check_access && !$site_setting->access('view', NULL, FALSE)) {
          return [];
        }
        $field = $site_setting->get($field_name);
        if ($field instanceof FieldItemListInterface) {
          $build = $this->entityTypeManager
            ->getViewBuilder('site_setting_entity')
            ->viewField($field, $view_mode);

          // Add the site setting to the cache tags.
          $cacheable_metadata = CacheableMetadata::createFromRenderArray($build);
          foreach ($entities as $site_setting) {
            $cacheable_metadata->addCacheableDependency($site_setting);
          }
          $cacheable_metadata->applyTo($build);
          return $build;
        }
      }
    }

    return [];
  }

  /**
   * {@inheritDoc}
   */
  public function siteSettingEntityByName(
    string $selector,
    ?string $langcode = NULL,
    bool $check_access = TRUE,
  ): ?SiteSettingEntityInterface {
    $storage = $this->entityTypeManager->getStorage('site_setting_entity');
    $entities = $storage->loadByProperties(['type' => $selector]);

    // Remove site setting entities that don't match the current domain context.
    $fallback_delta = NULL;
    $has_specific_context = FALSE;
    foreach ($entities as $delta => $entity) {
      if ($entity->get('domain_context')->isEmpty()) {
        $fallback_delta = $delta;
        continue;
      }

      $domain = $entity->get('domain_context')->entity;
      if ($domain->id() !== $this->domainNegotiator->getActiveDomain()->id()) {
        unset($entities[$delta]);
      }
      else {
        $has_specific_context = TRUE;
      }
    }

    // If a domain specific site setting is present, remove the fallback.
    if ($has_specific_context && $fallback_delta) {
      unset($entities[$fallback_delta]);
    }

    $entity = reset($entities);
    if ($entity) {

      // Check access on the entity.
      if ($check_access) {
        $access = $entity->access('view', NULL, TRUE);
      }
      else {
        $access = AccessResult::allowed();
      }
      if ($access->isAllowed()) {

        // Return the entity in the specified language code if exists and
        // allowed.
        $langcode = $langcode ?? $this->languageManager->getCurrentLanguage()->getId();
        if (
          $entity instanceof TranslatableInterface
          && $entity->language()->getId() !== $langcode
          && $entity->hasTranslation($langcode)
        ) {
          $translated_entity = $entity->getTranslation($langcode);
          if (
            $translated_entity
            && ($check_access || $translated_entity->access('view', NULL, FALSE))
          ) {
            return $translated_entity;
          }
        }

        // Return the original entity.
        return $entity;
      }
    }

    return NULL;
  }

}
